@@ -1,5 +1,6 @@
 #include <Rcpp.h>
 #include "shared.h"
+// [[Rcpp::interfaces(r, cpp)]]
 // [[Rcpp::plugins(cpp11)]]
 
 using std::pow;
@@ -14,21 +15,27 @@ using std::tan;
 using std::atan;
 
 
-inline double pdf_hcauchy(double x, double sigma, bool& throw_warning) {
+inline double logpdf_hcauchy(double x, double sigma,
+                             bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(sigma))
     return x+sigma;
+#endif
   if (sigma <= 0.0) {
     throw_warning = true;
     return NAN;
   }
   if (x < 0.0)
-    return 0.0;
-  return 2.0/(M_PI*(1.0 + pow(x/sigma, 2.0)))/sigma;
+    return R_NegInf;
+  // 2.0/(M_PI*(1.0 + pow(x/sigma, 2.0)))/sigma;
+  return LOG_2F - log(M_PI) - log1p(exp( (log(x)-log(sigma)) * 2.0 )) - log(sigma);
 }
 
-double cdf_hcauchy(double x, double sigma, bool& throw_warning) {
+inline double cdf_hcauchy(double x, double sigma, bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(sigma))
     return x+sigma;
+#endif
   if (sigma <= 0.0) {
     throw_warning = true;
     return NAN;
@@ -40,8 +47,10 @@ double cdf_hcauchy(double x, double sigma, bool& throw_warning) {
 
 inline double invcdf_hcauchy(double p, double sigma,
                              bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(p) || ISNAN(sigma))
     return p+sigma;
+#endif
   if (sigma <= 0.0 || !VALID_PROB(p)) {
     throw_warning = true;
     return NAN;
@@ -78,11 +87,11 @@ NumericVector cpp_dhcauchy(
   bool throw_warning = false;
   
   for (int i = 0; i < Nmax; i++)
-    p[i] = pdf_hcauchy(GETV(x, i), GETV(sigma, i),
-                       throw_warning);
+    p[i] = logpdf_hcauchy(GETV(x, i), GETV(sigma, i),
+                          throw_warning);
   
-  if (log_prob)
-    p = Rcpp::log(p);
+  if (!log_prob)
+    p = Rcpp::exp(p);
   
   if (throw_warning)
     Rcpp::warning("NaNs produced");
